@@ -5,6 +5,6 @@ Created on Fri Jul 30 13:22:34 2021
 @author: chang.sun
 """
 
-from .technical_analysis import *
-from .simpletest import *
+import sc_backtest.technical_analysis as technical_analysis
+from .simpletest import simpletest
 from .backtest import *
