@@ -5,12 +5,38 @@ Created on Thu Jul 29 17:14:23 2021
 @author: chang.sun
 """
 # %%
-
-
 import pandas as pd
 import numpy as np
-import matplotlib.pyplot as plt
 from copy import copy
+import matplotlib.pyplot as plt
+from .private_func import _preprocessing
+from .private_func import _get_report
+
+
+def get_report(indicator, ret):
+    """
+    :param indicator: pd.Series with DatatimeIndex
+    :param ret: pd.Series with DatatimeIndex
+    :return: report: pd.DataFrame
+    """
+    if len(indicator) != len(ret):
+        print(F'mismatched length of indicator, pnl, len(indicator) == {len(indicator)} len(ret) == {len(ret)}')
+        return
+
+    if not (isinstance(indicator, pd.Series)) or not (isinstance(ret, pd.Series)):
+        print('indicator and pnl must be pd.Series')
+        return
+
+    if not (isinstance(indicator.index, pd.core.indexes.datetimes.DatetimeIndex)) \
+            or not (isinstance(ret.index, pd.core.indexes.datetimes.DatetimeIndex)):
+        print('index of indicator and pnl must be pd.core.indexes.datetimes.DatetimeIndex')
+        return
+
+    report_all = _get_report(indicator, ret).rename('All').to_frame().T
+    report_sub = indicator.resample('Y').apply(lambda x: _get_report(x, ret.reindex(x.index))).unstack()
+    report_sub.index = report_sub.index.to_series().apply(lambda x: x.year)
+
+    return pd.concat([report_all, report_sub])
 
 
 class simpletest():
@@ -27,6 +53,8 @@ class simpletest():
         """
         x: pd.Series
         y: pd.Series
+        Sign-Trade: factor >(<) 0 -> indicator = 1(-1) * sign(corr)
+        Med-Trade: factor >(<) factor_mid -> indicator = 1(-1) * sign(corr)
         """
 
         if len(x) != len(y):
@@ -34,18 +62,20 @@ class simpletest():
             return
 
         corr = self.corr(x, y)
-        factor = x.copy()
+
+        factor, ret = _preprocessing(x, y)
+        factor_mid = np.median(factor)
 
         indicator1 = np.sign(factor) * 100 * np.sign(corr)
         label1 = 'Sign-Trade: '
 
-        indicator2 = factor / factor.abs().mean() * 100 * np.sign(corr)
-        label2 = 'Value-Trade: '
+        indicator2 = factor.apply(lambda m: 1 if m > factor_mid else -1) * 100 * np.sign(corr)
+        label2 = 'Med-Trade: '
 
-        pnl1 = y * indicator1
+        pnl1 = ret * indicator1
         sum_pnl1 = pnl1.cumsum()
 
-        pnl2 = y * indicator2
+        pnl2 = ret * indicator2
         sum_pnl2 = pnl2.cumsum()
 
         if self.is_day:
@@ -64,8 +94,8 @@ class simpletest():
         sharpe2 = day_pnl2.mean() / day_pnl2.std() * 16
 
         # year PnL
-        pnl_y1 = day_pnl1.mean() * 250
-        pnl_y2 = day_pnl2.mean() * 250
+        pnl_y1 = day_pnl1.mean() * 251
+        pnl_y2 = day_pnl2.mean() * 251
 
         # WR
         WR1 = day_pnl1[day_pnl1 >= 0].count() / len(day_pnl1)
@@ -90,7 +120,7 @@ class simpletest():
             ax.legend()
 
         if data_return:
-            return {'pnl_sign': sum_pnl1, 'pnl_value': sum_pnl2, 'delta_sign': indicator1, 'delta_value': indicator2}
+            return {'pnl_sign': sum_pnl1, 'pnl_med': sum_pnl2, 'delta_sign': indicator1, 'delta_med': indicator2}
 
         return
 
@@ -101,7 +131,7 @@ class simpletest():
         t_list: list: contains 1 thresold or 2 threholds
         """
         if not 1 <= len(t_list) <= 2:
-            print(F'mismatched length of t_list, len(t_list) == {len(t_list)} (t_list length should be 1 pr 2)')
+            print(F'incorrect length of t_list, len(t_list) == {len(t_list)} (t_list length should be 1 or 2)')
             return
 
         if len(x) != len(y):
@@ -109,7 +139,7 @@ class simpletest():
             return
 
         corr = self.corr(x, y)
-        factor = x.copy()
+        factor, ret = _preprocessing(x, y)
         indicator = factor.copy()
         if len(t_list) == 2:
             indicator.loc[factor > t_list[1]] = 1 * np.sign(corr)
@@ -119,7 +149,7 @@ class simpletest():
             indicator.loc[factor >= t_list[0]] = 1 * np.sign(corr)
             indicator.loc[factor < t_list[0]] = -1 * np.sign(corr)
 
-        pnl = y * indicator
+        pnl = ret * indicator
         sum_pnl = pnl.cumsum()
 
         if self.is_day:
@@ -318,18 +348,16 @@ class simpletest():
 
     def plot_composite(self, x, y, markout_periods=30, cdf_period2=5, fig=None, figsize=(20, 20), fig_return=False):
         """
-        x, y: pd.Series
+        x, y: prefer pd.Series
         x: factor
         y: future ret
         """
-        if (not isinstance(x, pd.Series)) or (not isinstance(y, pd.Series)):
-            print('x, y must be pd.Series!')
-            return
-
         if len(x) != len(y):
             print(F'mismatched length of x, y, len(x) == {len(x)} len(y) == {len(y)}')
             return
 
+        x, y = _preprocessing(x, y)
+
         if fig is None:
             fig = plt.figure(figsize=figsize)
 
